import { StrictUnicodeLine, StrictUnicodeChar } from '../../text/strictUnicode';
import { Style } from '../../text/style';
import { ArmT } from '../interfaces/arm/arm';
import { ArmPatternI, consumableArmCharsT } from '../interfaces/matrix/armPattern';
import { armWidthT } from '../interfaces/arm/armWidth';
export declare class ArmPattern implements ArmPatternI {
    firstChar: StrictUnicodeChar;
    otherChar: StrictUnicodeChar;
    lastChar: StrictUnicodeChar;
    style?: Style | undefined;
    constructor(firstChar: StrictUnicodeChar, otherChar: StrictUnicodeChar, lastChar: StrictUnicodeChar, style?: Style | undefined);
    /**
     * Generates line with drawn armPlainLine like: `╰───╸`
     * with repeating `this.other` character to fit `this.armWidth`
     *
     * @remarks
     *
     * During generation `first` have higher priority then `last`.
     * So `first` will be the only generated character if `this.armWidth = 1`.
     * Same way `other` have lowest priority. And appars only if `this.armWidth > 2`
     *
     * @param {armWidthT} armWidth
     * Width of generated armPlainLine. Measured in monospaced-character positions.
     * @returns {StrictUnicodeLine}
     * Drawn armPlainLine, wrapped in StrictUnicodeLine.
     *
     * @remarks
     *
     * Returning StrictUnicodeLine instead of raw string is to skip
     * safaty-checks in tree generator/renderer.
     *
     * @memberof BasicPattern
     */
    generateArmPlainLine(armWidth: armWidthT): StrictUnicodeLine;
    generateArm(armWidth: armWidthT): ArmT;
    static fromChars(chars: consumableArmCharsT, style?: Style): ArmPattern;
}
//# sourceMappingURL=armPattern.d.ts.map