"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const textContainer_1 = require("../../text/textContainer");
const armGeneratorChain_1 = require("./armGeneratorChain");
function renderMetaNode(node, maxWidth, renderer, logLineCallback) {
    const emptyChain = new armGeneratorChain_1.ArmGeneratorChain([]);
    return renderMetaNodeRecursive(node, emptyChain, maxWidth, renderer, logLineCallback);
}
exports.renderMetaNode = renderMetaNode;
function renderMetaNodeRecursive(node, parentChain, maxWidth, renderer, logLineCallback) {
    const hasChildren = (node.children.length > 0);
    const generateArmParameters = {
        node,
        childNum: null,
        lineNum: 0,
        isLastLine: false,
        lineOfKnotNum: 0,
        isLastLineOfKnot: false,
    };
    const currentChainElement = new armGeneratorChain_1.ArmGeneratorChainElement(node.armGenerator, node.armWidthGenerator, generateArmParameters);
    // iterating through leaf lines
    let currentArm;
    let parentArmChain;
    let armChain;
    let arm;
    let currentLineMaxWidth;
    let leafLineWrapGenerator;
    let generatorResult;
    let isLastLineOfLeaf;
    do {
        currentArm = currentChainElement.generateArm();
        parentArmChain = parentChain.generateArmChain();
        armChain = parentArmChain.concat(currentArm);
        arm = new textContainer_1.NonatomicTextContainer(armChain);
        currentLineMaxWidth = maxWidth - arm.calcSize().width.first;
        if (!leafLineWrapGenerator) {
            leafLineWrapGenerator = node.leaf.managedWrap(currentLineMaxWidth);
        }
        generatorResult = leafLineWrapGenerator.next(currentLineMaxWidth);
        isLastLineOfLeaf = !!generatorResult.done;
        parentChain.elements.forEach((parentElement) => {
            const isLastChildOfParent = parentElement.parameters.childNum === parentElement.parameters.node.children.length - 1;
            if (isLastChildOfParent && isLastLineOfLeaf) {
                parentElement.parameters.isLastLine = true;
                parentElement.parameters.isLastLineOfKnot = true;
            }
        });
        if (!hasChildren && isLastLineOfLeaf) {
            currentChainElement.parameters.isLastLine = true;
            currentChainElement.parameters.isLastLineOfKnot = true;
        }
        const wrappedLineContainer = generatorResult.value;
        const lineToRender = new textContainer_1.NonatomicTextContainer([arm, wrappedLineContainer]);
        const rendered = renderer.render(lineToRender);
        logLineCallback(rendered.rendered, rendered.trailingEol);
        parentChain.elements.forEach((parentElement) => {
            parentElement.parameters.lineNum++;
            parentElement.parameters.lineOfKnotNum++;
        });
        currentChainElement.parameters.lineNum++;
        currentChainElement.parameters.lineOfKnotNum++;
    } while (!generatorResult.done);
    const chain = new armGeneratorChain_1.ArmGeneratorChain(parentChain.elements.concat(currentChainElement));
    // iterating through children
    node.children.forEach((childNode, childNodeId) => {
        currentChainElement.parameters.childNum = childNodeId;
        currentChainElement.parameters.lineOfKnotNum = 0;
        currentChainElement.parameters.isLastLineOfKnot = false;
        renderMetaNodeRecursive(childNode, chain, maxWidth, renderer, logLineCallback);
    });
}
//# sourceMappingURL=render.js.map