"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const armPatternMatrix_1 = require("../meta/matrix/armPatternMatrix");
const armWidthMatrix_1 = require("../meta/matrix/armWidthMatrix");
const matrixDrivenArmGenerator_1 = require("../meta/matrix/matrixDrivenArmGenerator");
const defaultThemeOld_1 = require("./defaultThemeOld");
const nodeType_1 = require("./interfaces/nodeType");
const deadArmPatternMatrix = armPatternMatrix_1.ArmPatternMatrix.fromArray([
    '   ', '   ', '   ',
    '   ', '?  ', '?  ',
    '???', '?  ', '?  ',
    '???', '?  ', '?  ',
    '???', '   ', '   ',
], defaultThemeOld_1.theme.style.branch[nodeType_1.NodeBroadTypeEnum.Dead]);
const singleArmPatternMatrix = armPatternMatrix_1.ArmPatternMatrix.fromArray([
    '   ', '   ', '   ',
    '   ', '?  ', '?  ',
    '???', '?  ', '?  ',
    '???', '?  ', '?  ',
    '???', '   ', '   ',
], defaultThemeOld_1.theme.style.branch[nodeType_1.NodeBroadTypeEnum.Single]);
const iterableArmPatternMatrix = armPatternMatrix_1.ArmPatternMatrix.fromArray([
    '   ', '   ', '   ',
    '   ', '│  ', '│  ',
    '├─╸', '│  ', '│  ',
    '├─╸', '│  ', '│  ',
    '└─╸', '   ', '   ',
], defaultThemeOld_1.theme.style.branch[nodeType_1.NodeBroadTypeEnum.Iterable]);
const enumerableArmPatternMatrix = armPatternMatrix_1.ArmPatternMatrix.fromArray([
    '   ', '   ', '   ',
    '   ', '│  ', '│  ',
    '├─╴', '│  ', '│  ',
    '├─╴', '│  ', '│  ',
    '╰─╴', '   ', '   ',
], defaultThemeOld_1.theme.style.branch[nodeType_1.NodeBroadTypeEnum.Enumerable]);
const armWidth = { preSpace: 1, postSpace: 1, arm: 4 };
const smallArmWidth = { preSpace: 1, postSpace: 1, arm: 1 };
const armWidthMatrix = armWidthMatrix_1.ArmWidthMatrix.fromArray([
    0, 0, 0,
    0, smallArmWidth, smallArmWidth,
    armWidth, armWidth, armWidth,
    armWidth, armWidth, armWidth,
    armWidth, armWidth, armWidth,
]);
const dead = {
    armGenerator: new matrixDrivenArmGenerator_1.MatrixDrivenArmGenerator(deadArmPatternMatrix),
    armWidthGenerator: new matrixDrivenArmGenerator_1.MatrixDrivenArmWidthGenerator(armWidthMatrix),
};
const single = {
    armGenerator: new matrixDrivenArmGenerator_1.MatrixDrivenArmGenerator(singleArmPatternMatrix),
    armWidthGenerator: new matrixDrivenArmGenerator_1.MatrixDrivenArmWidthGenerator(armWidthMatrix),
};
const iterable = {
    armGenerator: new matrixDrivenArmGenerator_1.MatrixDrivenArmGenerator(iterableArmPatternMatrix),
    armWidthGenerator: new matrixDrivenArmGenerator_1.MatrixDrivenArmWidthGenerator(armWidthMatrix),
};
const enumerable = {
    armGenerator: new matrixDrivenArmGenerator_1.MatrixDrivenArmGenerator(enumerableArmPatternMatrix),
    armWidthGenerator: new matrixDrivenArmGenerator_1.MatrixDrivenArmWidthGenerator(armWidthMatrix),
};
exports.templateDictionary = {
    [nodeType_1.NodeBroadTypeEnum.Dead]: dead,
    [nodeType_1.NodeBroadTypeEnum.Single]: single,
    [nodeType_1.NodeBroadTypeEnum.Iterable]: iterable,
    [nodeType_1.NodeBroadTypeEnum.Enumerable]: enumerable,
};
//# sourceMappingURL=defaultTemplateOld.js.map