import { prebuildedThemeT } from './interfaces/prebuildedTheme';
import { themeT } from './interfaces/theme';
import { printTreeOptionsT, stringifyTreeOptionsT, oneshotPrintTreeOptionsT, oneshotStringifyTreeOptionsT } from './interfaces/general';
export declare class Pretty {
    protected readonly theme: prebuildedThemeT;
    constructor(themeStack?: themeT[]);
    /**
   * Print formatted tree-representation of any javacript data. Use options
   * to customize output.
   *
   * @remarks
   *
   * Use `options.logLineCallback` for custom `console.log` handler, or
   * `pretty.stringifyTree` to render tree to a `string`.
   *
   * @remarks
   *
   * Cycle-referencing is allowed, while capping output
   * with finite `options.maxLevel` and `options.maxItemsPerLevel`.
   *
   * @export
   * @param {*} tree
   * @param {printTreeOptionsT} [options]
   */
    print(tree: any, options?: printTreeOptionsT): void;
    /**
   * Render formatted tree-representation of any javacript data to string.
   * Use options to customize output.
   *
   * @remarks
   *
   * Works same as `pretty.print`, but use predefined `logLineCallback`
   * for dumping output to `string`. Consider Using `pretty.print` if
   * you need a per-line callback model, or if you just want a simple oneliner
   * with default `options.logLineCallback`
   *
   * @remarks
   *
   * Cycle-referencing is allowed, while capping output
   * with finite `options.maxLevel` and `options.maxItemsPerLevel`.
   *
   *
   * @export
   * @param {*} tree
   * @param {stringifyTreeOptionsT} [options]
   * @returns {string}
   */
    stringify(tree: any, options?: stringifyTreeOptionsT): string;
    static print(tree: any, options?: oneshotPrintTreeOptionsT): void;
    static stringify(tree: any, options?: oneshotStringifyTreeOptionsT): string;
}
//# sourceMappingURL=pretty.d.ts.map