"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const prebuildTheme_1 = require("./theme/prebuildTheme");
const fallback_1 = require("./theme/fallback");
const buildMetaTree_1 = require("./buildMetaTree/buildMetaTree");
const render_1 = require("../meta/render/render");
const defaultSettings_1 = require("./defaultSettings");
class Pretty {
    constructor(themeStack) {
        this.theme = prebuildTheme_1.prebuildTheme(themeStack || [], fallback_1.fallback);
    }
    /**
   * Print formatted tree-representation of any javacript data. Use options
   * to customize output.
   *
   * @remarks
   *
   * Use `options.logLineCallback` for custom `console.log` handler, or
   * `pretty.stringifyTree` to render tree to a `string`.
   *
   * @remarks
   *
   * Cycle-referencing is allowed, while capping output
   * with finite `options.maxLevel` and `options.maxItemsPerLevel`.
   *
   * @export
   * @param {*} tree
   * @param {printTreeOptionsT} [options]
   */
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    print(tree, options) {
        const settings = Object.assign({}, defaultSettings_1.defaultSettings, options);
        const metaTree = buildMetaTree_1.buildMetaTree(tree, this.theme, settings);
        render_1.renderMetaNode(metaTree, settings.maxLineWidth, settings.renderer, settings.logLineCallback);
    }
    /**
   * Render formatted tree-representation of any javacript data to string.
   * Use options to customize output.
   *
   * @remarks
   *
   * Works same as `pretty.print`, but use predefined `logLineCallback`
   * for dumping output to `string`. Consider Using `pretty.print` if
   * you need a per-line callback model, or if you just want a simple oneliner
   * with default `options.logLineCallback`
   *
   * @remarks
   *
   * Cycle-referencing is allowed, while capping output
   * with finite `options.maxLevel` and `options.maxItemsPerLevel`.
   *
   *
   * @export
   * @param {*} tree
   * @param {stringifyTreeOptionsT} [options]
   * @returns {string}
   */
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    stringify(tree, options) {
        let stringBuffer = '';
        const printTreeOptions = Object.assign({}, options || {}, {
            logLineCallback(line, trailingEol) { stringBuffer += line + trailingEol; },
        });
        this.print(tree, printTreeOptions);
        return stringBuffer;
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    static print(tree, options) {
        const settings = Object.assign({}, defaultSettings_1.defaultSettings, options);
        const oneshotPretty = new Pretty(settings.themeStack);
        oneshotPretty.print(tree, settings);
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    static stringify(tree, options) {
        const settings = Object.assign({}, defaultSettings_1.defaultSettings, options);
        const oneshotPretty = new Pretty(settings.themeStack);
        return oneshotPretty.stringify(tree, settings);
    }
}
exports.Pretty = Pretty;
//# sourceMappingURL=pretty.js.map