"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const textContainer_1 = require("../../text/textContainer");
const strictUnicode_1 = require("../../text/strictUnicode");
const matrixDrivenArmGenerator_1 = require("../../meta/matrix/matrixDrivenArmGenerator");
const armPatternMatrix_1 = require("../../meta/matrix/armPatternMatrix");
const armWidthMatrix_1 = require("../../meta/matrix/armWidthMatrix");
const nodeType_1 = require("../interfaces/nodeType");
const typeDependentDictionary_1 = require("../typeDependentDictionary");
function mapFilterUndefined(array, callbackfn) {
    // typecheck workaround
    const mapped = array.map(callbackfn);
    function filterFn(element) {
        return element !== undefined;
    }
    const filtered = mapped.filter(filterFn);
    return filtered;
}
function collapseStack(stack) {
    let collapsed;
    if (stack.length > 0) {
        collapsed = Object.assign({}, ...stack);
    }
    return collapsed;
}
function mapFilterCollapseStack(stack, callbackfn) {
    const filteredStack = mapFilterUndefined(stack, callbackfn);
    const collapsed = collapseStack(filteredStack);
    return collapsed;
}
function collapseNodeThemeStack(nodeThemeStack) {
    // root
    const flatCollapsed = Object.assign({}, ...nodeThemeStack);
    // arm
    flatCollapsed.arm = mapFilterCollapseStack(nodeThemeStack, (nodeTheme) => nodeTheme.arm);
    // icon
    flatCollapsed.icon = mapFilterCollapseStack(nodeThemeStack, (nodeTheme) => nodeTheme.icon);
    if (flatCollapsed.icon) {
        // icon.predelimiter
        flatCollapsed.icon.predelimiter = mapFilterCollapseStack(nodeThemeStack, (nodeTheme) => { var _a; return (_a = nodeTheme.icon) === null || _a === void 0 ? void 0 : _a.predelimiter; });
        // icon.prefix
        flatCollapsed.icon.prefix = mapFilterCollapseStack(nodeThemeStack, (nodeTheme) => { var _a; return (_a = nodeTheme.icon) === null || _a === void 0 ? void 0 : _a.prefix; });
        // icon.content
        flatCollapsed.icon.content = mapFilterCollapseStack(nodeThemeStack, (nodeTheme) => { var _a; return (_a = nodeTheme.icon) === null || _a === void 0 ? void 0 : _a.content; });
        // icon.postfix
        flatCollapsed.icon.postfix = mapFilterCollapseStack(nodeThemeStack, (nodeTheme) => { var _a; return (_a = nodeTheme.icon) === null || _a === void 0 ? void 0 : _a.postfix; });
        // icon.postdelimiter
        flatCollapsed.icon.postdelimiter = mapFilterCollapseStack(nodeThemeStack, (nodeTheme) => { var _a; return (_a = nodeTheme.icon) === null || _a === void 0 ? void 0 : _a.postdelimiter; });
    }
    // key
    flatCollapsed.key = mapFilterCollapseStack(nodeThemeStack, (nodeTheme) => nodeTheme.key);
    if (flatCollapsed.key) {
        // key.predelimiter
        flatCollapsed.key.predelimiter = mapFilterCollapseStack(nodeThemeStack, (nodeTheme) => { var _a; return (_a = nodeTheme.key) === null || _a === void 0 ? void 0 : _a.predelimiter; });
        // key.prefix
        flatCollapsed.key.prefix = mapFilterCollapseStack(nodeThemeStack, (nodeTheme) => { var _a; return (_a = nodeTheme.key) === null || _a === void 0 ? void 0 : _a.prefix; });
        // key.content
        flatCollapsed.key.content = mapFilterCollapseStack(nodeThemeStack, (nodeTheme) => { var _a; return (_a = nodeTheme.key) === null || _a === void 0 ? void 0 : _a.content; });
        // key.postfix
        flatCollapsed.key.postfix = mapFilterCollapseStack(nodeThemeStack, (nodeTheme) => { var _a; return (_a = nodeTheme.key) === null || _a === void 0 ? void 0 : _a.postfix; });
        // key.postdelimiter
        flatCollapsed.key.postdelimiter = mapFilterCollapseStack(nodeThemeStack, (nodeTheme) => { var _a; return (_a = nodeTheme.key) === null || _a === void 0 ? void 0 : _a.postdelimiter; });
    }
    // value
    flatCollapsed.value = mapFilterCollapseStack(nodeThemeStack, (nodeTheme) => nodeTheme.value);
    if (flatCollapsed.value) {
        // value.predelimiter
        flatCollapsed.value.predelimiter = mapFilterCollapseStack(nodeThemeStack, (nodeTheme) => { var _a; return (_a = nodeTheme.value) === null || _a === void 0 ? void 0 : _a.predelimiter; });
        // value.prefix
        flatCollapsed.value.prefix = mapFilterCollapseStack(nodeThemeStack, (nodeTheme) => { var _a; return (_a = nodeTheme.value) === null || _a === void 0 ? void 0 : _a.prefix; });
        // value.content
        flatCollapsed.value.content = mapFilterCollapseStack(nodeThemeStack, (nodeTheme) => { var _a; return (_a = nodeTheme.value) === null || _a === void 0 ? void 0 : _a.content; });
        // value.postfix
        flatCollapsed.value.postfix = mapFilterCollapseStack(nodeThemeStack, (nodeTheme) => { var _a; return (_a = nodeTheme.value) === null || _a === void 0 ? void 0 : _a.postfix; });
        // value.postdelimiter
        flatCollapsed.value.postdelimiter = mapFilterCollapseStack(nodeThemeStack, (nodeTheme) => { var _a; return (_a = nodeTheme.value) === null || _a === void 0 ? void 0 : _a.postdelimiter; });
    }
    // info
    flatCollapsed.info = mapFilterCollapseStack(nodeThemeStack, (nodeTheme) => nodeTheme.info);
    if (flatCollapsed.info) {
        // info.predelimiter
        flatCollapsed.info.predelimiter = mapFilterCollapseStack(nodeThemeStack, (nodeTheme) => { var _a; return (_a = nodeTheme.info) === null || _a === void 0 ? void 0 : _a.predelimiter; });
        // info.prefix
        flatCollapsed.info.prefix = mapFilterCollapseStack(nodeThemeStack, (nodeTheme) => { var _a; return (_a = nodeTheme.info) === null || _a === void 0 ? void 0 : _a.prefix; });
        // info.content
        flatCollapsed.info.content = mapFilterCollapseStack(nodeThemeStack, (nodeTheme) => { var _a; return (_a = nodeTheme.info) === null || _a === void 0 ? void 0 : _a.content; });
        // info.postfix
        flatCollapsed.info.postfix = mapFilterCollapseStack(nodeThemeStack, (nodeTheme) => { var _a; return (_a = nodeTheme.info) === null || _a === void 0 ? void 0 : _a.postfix; });
        // info.postdelimiter
        flatCollapsed.info.postdelimiter = mapFilterCollapseStack(nodeThemeStack, (nodeTheme) => { var _a; return (_a = nodeTheme.info) === null || _a === void 0 ? void 0 : _a.postdelimiter; });
    }
    // remark
    flatCollapsed.remark = mapFilterCollapseStack(nodeThemeStack, (nodeTheme) => nodeTheme.remark);
    if (flatCollapsed.remark) {
        // remark.predelimiter
        flatCollapsed.remark.predelimiter = mapFilterCollapseStack(nodeThemeStack, (nodeTheme) => { var _a; return (_a = nodeTheme.remark) === null || _a === void 0 ? void 0 : _a.predelimiter; });
        // remark.prefix
        flatCollapsed.remark.prefix = mapFilterCollapseStack(nodeThemeStack, (nodeTheme) => { var _a; return (_a = nodeTheme.remark) === null || _a === void 0 ? void 0 : _a.prefix; });
        // remark.content
        flatCollapsed.remark.content = mapFilterCollapseStack(nodeThemeStack, (nodeTheme) => { var _a; return (_a = nodeTheme.remark) === null || _a === void 0 ? void 0 : _a.content; });
        // remark.postfix
        flatCollapsed.remark.postfix = mapFilterCollapseStack(nodeThemeStack, (nodeTheme) => { var _a; return (_a = nodeTheme.remark) === null || _a === void 0 ? void 0 : _a.postfix; });
        // remark.postdelimiter
        flatCollapsed.remark.postdelimiter = mapFilterCollapseStack(nodeThemeStack, (nodeTheme) => { var _a; return (_a = nodeTheme.remark) === null || _a === void 0 ? void 0 : _a.postdelimiter; });
    }
    return flatCollapsed;
}
// node theme prebuild
function prebuildItemTheme(itemTheme) {
    if (itemTheme) {
        if (itemTheme.visibility === false) {
            return false;
        }
        else {
            const style = itemTheme.style;
            return { style };
        }
    }
    return undefined;
}
function prebuildPredefinableItemTheme(itemTheme) {
    if (itemTheme) {
        if (itemTheme.visibility === false) {
            return false;
        }
        else {
            if (itemTheme.line) {
                const container = new textContainer_1.AtomicTextContainer(new strictUnicode_1.StrictUnicodeLine(itemTheme.line), itemTheme.style);
                return { container };
            }
        }
    }
    return undefined;
}
function prebuildNodeSuffixedSemipredefinableItemTheme(suffixedItemTheme) {
    if (suffixedItemTheme) {
        if (suffixedItemTheme.visibility === false) {
            return false;
        }
        let prebuidedPredelimiter;
        let prebuidedPrefix;
        let prebuidedContent;
        let prebuidedPostfix;
        let prebuidedPostdelimiter;
        // TODO: deep fallback to prebuided-fallback subparts instead just not creating it if no theme for it
        if (suffixedItemTheme.predelimiter) {
            prebuidedPredelimiter = prebuildPredefinableItemTheme(suffixedItemTheme.predelimiter) || undefined;
        }
        if (suffixedItemTheme.prefix) {
            prebuidedPrefix = prebuildPredefinableItemTheme(suffixedItemTheme.prefix) || undefined;
        }
        if (suffixedItemTheme.content) {
            prebuidedContent = prebuildItemTheme(suffixedItemTheme.content) || undefined;
        }
        if (suffixedItemTheme.postfix) {
            prebuidedPostfix = prebuildPredefinableItemTheme(suffixedItemTheme.postfix) || undefined;
        }
        if (suffixedItemTheme.postdelimiter) {
            prebuidedPostdelimiter = prebuildPredefinableItemTheme(suffixedItemTheme.postdelimiter) || undefined;
        }
        const style = suffixedItemTheme.style;
        const prebuilded = { predelimiter: prebuidedPredelimiter, prefix: prebuidedPrefix, content: prebuidedContent, postfix: prebuidedPostfix, postdelimiter: prebuidedPostdelimiter, style };
        return prebuilded;
    }
    return undefined;
}
function prebuildNodeSuffixedPredefinableItemTheme(suffixedItemTheme) {
    if (suffixedItemTheme) {
        if (suffixedItemTheme.visibility === false) {
            return false;
        }
        let prebuidedPredelimiter;
        let prebuidedPrefix;
        let prebuidedContent;
        let prebuidedPostfix;
        let prebuidedPostdelimiter;
        // TODO: deep fallback to prebuided-fallback subparts instead just not creating it if no theme for it
        if (suffixedItemTheme.predelimiter) {
            prebuidedPredelimiter = prebuildPredefinableItemTheme(suffixedItemTheme.predelimiter) || undefined;
        }
        if (suffixedItemTheme.prefix) {
            prebuidedPrefix = prebuildPredefinableItemTheme(suffixedItemTheme.prefix) || undefined;
        }
        if (suffixedItemTheme.content) {
            prebuidedContent = prebuildPredefinableItemTheme(suffixedItemTheme.content) || undefined;
        }
        if (suffixedItemTheme.postfix) {
            prebuidedPostfix = prebuildPredefinableItemTheme(suffixedItemTheme.postfix) || undefined;
        }
        if (suffixedItemTheme.postdelimiter) {
            prebuidedPostdelimiter = prebuildPredefinableItemTheme(suffixedItemTheme.postdelimiter) || undefined;
        }
        const style = suffixedItemTheme.style;
        const prebuilded = { predelimiter: prebuidedPredelimiter, prefix: prebuidedPrefix, content: prebuidedContent, postfix: prebuidedPostfix, postdelimiter: prebuidedPostdelimiter, style };
        return prebuilded;
    }
    return undefined;
}
function prebuildArmNodeTheme(nodeArmTheme) {
    if (nodeArmTheme) {
        const armStyle = nodeArmTheme.style;
        if (nodeArmTheme.width !== undefined) {
            const armWidth = nodeArmTheme.width;
            const spacedArmWidth = { preSpace: 1, arm: armWidth, postSpace: 1 };
            const fertileLeafNonfirstLineSpacedArmWidth = { preSpace: 1, arm: 1, postSpace: 1 };
            const commonChars = nodeArmTheme.commonChars;
            if (commonChars) {
                return {
                    armGenerator: new matrixDrivenArmGenerator_1.MatrixDrivenArmGenerator(armPatternMatrix_1.ArmPatternMatrix.fromCommonChars(commonChars[0], commonChars[1], commonChars[2], armStyle)),
                    armWidthGenerator: new matrixDrivenArmGenerator_1.MatrixDrivenArmWidthGenerator(armWidthMatrix_1.ArmWidthMatrix.fromCommonWidth(spacedArmWidth, fertileLeafNonfirstLineSpacedArmWidth)),
                };
            }
        }
    }
    return undefined;
}
function prebuildNodeTheme(themeStack, fallback) {
    const nodeTheme = collapseNodeThemeStack(themeStack);
    const prebuilded = {
        // TODO strictly check for false & hide if it is (check root nodeTheme.visibility too)
        arm: prebuildArmNodeTheme(nodeTheme.arm) || fallback.arm,
        icon: prebuildNodeSuffixedPredefinableItemTheme(nodeTheme.icon) || fallback.icon,
        key: prebuildNodeSuffixedSemipredefinableItemTheme(nodeTheme.key) || fallback.key,
        value: prebuildNodeSuffixedSemipredefinableItemTheme(nodeTheme.value) || fallback.value,
        info: prebuildNodeSuffixedSemipredefinableItemTheme(nodeTheme.info) || fallback.info,
        remark: prebuildNodeSuffixedSemipredefinableItemTheme(nodeTheme.remark) || fallback.remark,
        style: nodeTheme.style,
    };
    return prebuilded;
}
exports.prebuildNodeTheme = prebuildNodeTheme;
function prebuildTheme(themeStack, fallback) {
    const prebuildedThemePartial = {};
    const broadKeys = Object.values(nodeType_1.NodeBroadTypeEnum);
    for (const broadKeySting of broadKeys) {
        const broadType = broadKeySting;
        const prebuildedThemeFinePartial = {};
        let fineEnum;
        switch (broadType) {
            case nodeType_1.NodeBroadTypeEnum.Dead:
                fineEnum = nodeType_1.DeadNodeFineTypeEnum;
                break;
            case nodeType_1.NodeBroadTypeEnum.Enumerable:
                fineEnum = nodeType_1.EnumerableNodeFineTypeEnum;
                break;
            case nodeType_1.NodeBroadTypeEnum.Iterable:
                fineEnum = nodeType_1.IterableNodeFineTypeEnum;
                break;
            case nodeType_1.NodeBroadTypeEnum.Single:
                fineEnum = nodeType_1.SingleNodeFineTypeEnum;
                break;
        }
        const fineKeys = Object.values(fineEnum);
        for (const fineTypeSting of fineKeys) {
            const fineType = fineTypeSting;
            const typeTuple = [broadType, fineType];
            const nodeThemeStack = [];
            themeStack.forEach((theme) => {
                if (theme.global) {
                    nodeThemeStack.push(theme.global);
                }
                if (theme.broad) {
                    const broad = typeDependentDictionary_1.getFromTypeDependentBroadOnlyPartialDictionary(theme.broad, broadType);
                    if (broad) {
                        nodeThemeStack.push(broad);
                    }
                }
                if (theme.fine) {
                    const fine = typeDependentDictionary_1.getFromTypeDependentPartialDictionary(theme.fine, typeTuple);
                    if (fine) {
                        nodeThemeStack.push(fine);
                    }
                }
                theme.global;
            });
            prebuildedThemeFinePartial[fineType] = prebuildNodeTheme(nodeThemeStack, fallback);
        }
        prebuildedThemePartial[broadType] = prebuildedThemeFinePartial;
    }
    return prebuildedThemePartial;
}
exports.prebuildTheme = prebuildTheme;
//# sourceMappingURL=prebuild.js.map