"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const style_1 = require("../../../text/style");
const safeAnsiPalette = {
    color: {
        info: { r: 162, g: 138, b: 49 },
        infoDelimiter: { r: 162 / 2, g: 138 / 2, b: 49 / 2 },
        remark: { r: 255, g: 128, b: 0 },
        value: { r: 225, g: 225, b: 225 },
        unvalue: { r: 225, g: 225, b: 128 },
        keyDelimiter: { r: 64, g: 64, b: 64 },
        key: { r: 128, g: 128, b: 128 },
        enumerable: {
            arm: { r: 0 + 16, g: 128 - 48, b: 0 + 16 },
            keyBrackets: { r: 64, g: 255 - 48, b: 64 },
        },
        iterable: {
            arm: { r: 0 + 32, g: 0 + 48, b: 128 - 16 },
            keyBrackets: { r: 64 - 16, g: 64 + 0, b: 255 },
        },
        single: {
            arm: { r: 64, g: 64, b: 64 },
            keyBrackets: { r: 128, g: 128, b: 128 },
        },
        dead: {
            arm: { r: 128, g: 0, b: 0 },
            keyBrackets: { r: 255, g: 64, b: 64 },
            elipsis: { r: 128, g: 128, b: 128 },
            circularReference: { r: 192, g: 64, b: 64 },
        },
    },
};
function styleIfNeed(foreground, background, styleSwatches) {
    if (foreground || background || styleSwatches) {
        return new style_1.Style(foreground, background, styleSwatches);
    }
    else {
        return undefined;
    }
}
function buildTheme(palette) {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r, _s, _t, _u, _v, _w, _x, _y, _z, _0, _1, _2, _3, _4, _5, _6, _7, _8, _9;
    const theme = {
        global: {
            visibility: true,
            arm: {
                //commonChars: ['|--', '|  ', 'L--',],
                commonChars: ['├──', '│  ', '└──',],
                width: { preSpace: 0, arm: 4, postSpace: 0 },
            },
            key: {
                style: styleIfNeed((_a = palette.color) === null || _a === void 0 ? void 0 : _a.key, undefined),
                postdelimiter: {
                    line: ': ',
                    style: styleIfNeed((_b = palette.color) === null || _b === void 0 ? void 0 : _b.keyDelimiter),
                }
            },
            value: {
                style: styleIfNeed((_c = palette.color) === null || _c === void 0 ? void 0 : _c.value),
            },
            info: {
                predelimiter: {
                    line: ' / ',
                    style: styleIfNeed((_d = palette.color) === null || _d === void 0 ? void 0 : _d.infoDelimiter, undefined),
                },
                style: styleIfNeed((_e = palette.color) === null || _e === void 0 ? void 0 : _e.info, undefined),
            },
            remark: {
                predelimiter: {
                    line: ' * ',
                },
                style: styleIfNeed((_f = palette.color) === null || _f === void 0 ? void 0 : _f.remark),
            },
        },
        broad: {
            enumerable: {
                arm: {
                    style: styleIfNeed((_h = (_g = palette.color) === null || _g === void 0 ? void 0 : _g.enumerable) === null || _h === void 0 ? void 0 : _h.arm),
                },
                key: {
                    prefix: {
                        line: '{',
                        style: styleIfNeed((_k = (_j = palette.color) === null || _j === void 0 ? void 0 : _j.enumerable) === null || _k === void 0 ? void 0 : _k.keyBrackets, undefined),
                    },
                    postfix: {
                        line: '}',
                        style: styleIfNeed((_m = (_l = palette.color) === null || _l === void 0 ? void 0 : _l.enumerable) === null || _m === void 0 ? void 0 : _m.keyBrackets, undefined),
                    },
                },
            },
            iterable: {
                arm: {
                    style: styleIfNeed((_p = (_o = palette.color) === null || _o === void 0 ? void 0 : _o.iterable) === null || _p === void 0 ? void 0 : _p.arm),
                },
                key: {
                    prefix: {
                        line: '[',
                        style: styleIfNeed((_r = (_q = palette.color) === null || _q === void 0 ? void 0 : _q.iterable) === null || _r === void 0 ? void 0 : _r.keyBrackets, undefined),
                    },
                    postfix: {
                        line: ']',
                        style: styleIfNeed((_t = (_s = palette.color) === null || _s === void 0 ? void 0 : _s.iterable) === null || _t === void 0 ? void 0 : _t.keyBrackets, undefined),
                    },
                },
            },
            single: {
                arm: {
                    style: styleIfNeed((_v = (_u = palette.color) === null || _u === void 0 ? void 0 : _u.single) === null || _v === void 0 ? void 0 : _v.arm),
                },
                key: {
                    prefix: {
                        line: '<',
                        style: styleIfNeed((_x = (_w = palette.color) === null || _w === void 0 ? void 0 : _w.single) === null || _x === void 0 ? void 0 : _x.keyBrackets, undefined),
                    },
                    postfix: {
                        line: '>',
                        style: styleIfNeed((_z = (_y = palette.color) === null || _y === void 0 ? void 0 : _y.single) === null || _z === void 0 ? void 0 : _z.keyBrackets, undefined),
                    },
                },
            },
            dead: {
                arm: {
                    style: styleIfNeed((_1 = (_0 = palette.color) === null || _0 === void 0 ? void 0 : _0.dead) === null || _1 === void 0 ? void 0 : _1.arm),
                },
                key: {
                    prefix: {
                        line: '>',
                        style: styleIfNeed((_3 = (_2 = palette.color) === null || _2 === void 0 ? void 0 : _2.dead) === null || _3 === void 0 ? void 0 : _3.keyBrackets, undefined),
                    },
                    postfix: {
                        line: '<',
                        style: styleIfNeed((_5 = (_4 = palette.color) === null || _4 === void 0 ? void 0 : _4.dead) === null || _5 === void 0 ? void 0 : _5.keyBrackets, undefined),
                    },
                },
            },
        },
        fine: {
            dead: {
                elipsis: {
                    icon: {
                        content: {
                            line: '...',
                            style: styleIfNeed((_7 = (_6 = palette.color) === null || _6 === void 0 ? void 0 : _6.dead) === null || _7 === void 0 ? void 0 : _7.elipsis),
                        }
                    }
                },
                circularReference: {
                    icon: {
                        content: {
                            line: '>∞<',
                            style: styleIfNeed((_9 = (_8 = palette.color) === null || _8 === void 0 ? void 0 : _8.dead) === null || _9 === void 0 ? void 0 : _9.circularReference),
                        }
                    }
                },
            }
        }
    };
    return theme;
}
exports.legacy = {
    standart: buildTheme(safeAnsiPalette),
};
//# sourceMappingURL=legacy.js.map