"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const style_1 = require("../../../text/style");
const rgbUtils_1 = require("../rgbUtils");
const standartPalette = {
    enumerable: {
        color: { r: 50, g: 140, b: 50 },
        counterColor: { r: 0, g: 0, b: 0 },
    },
    iterable: {
        color: { r: 100, g: 100, b: 200 },
        counterColor: { r: 0, g: 0, b: 0 },
    },
    single: {
        color: { r: 40, g: 40, b: 40 },
        counterColor: { r: 96, g: 96, b: 96 },
    },
    dead: {
        color: { r: 200, g: 100, b: 100 },
        counterColor: { r: 0, g: 0, b: 0 },
    },
    dim: {
        gamma: 0.6,
        vibrance: 0.6
    },
    emphasis: {
        gamma: 0.75,
        vibrance: 1.2
    },
};
const cutePalette = {
    enumerable: {
        color: { r: 45, g: 145, b: 65 },
        counterColor: { r: 0, g: 0, b: 0 },
    },
    iterable: {
        color: { r: 70, g: 130, b: 180 },
        counterColor: { r: 0, g: 0, b: 0 },
    },
    single: {
        color: { r: 40, g: 42, b: 44 },
        counterColor: { r: 97, g: 96, b: 95 },
    },
    dead: {
        color: { r: 200, g: 140, b: 60 },
        counterColor: { r: 0, g: 0, b: 0 },
    },
    dim: {
        gamma: 0.6,
        vibrance: 0.6
    },
    emphasis: {
        gamma: 0.75,
        vibrance: 1.2
    },
};
const alternativePalette = {
    enumerable: {
        color: rgbUtils_1.gainLuminosity({ r: 65, g: 56, b: 99 }, 1.25),
        counterColor: { r: 0, g: 0, b: 0 },
    },
    iterable: {
        color: rgbUtils_1.gainLuminosity({ r: 99, g: 60, b: 40 }, 1.2),
        counterColor: { r: 0, g: 0, b: 0 },
    },
    single: {
        color: { r: 35, g: 36, b: 32 },
        counterColor: { r: 110, g: 120, b: 105 },
    },
    dead: {
        color: { r: 140, g: 33, b: 11 },
        counterColor: { r: 0, g: 0, b: 0 },
    },
    dim: {
        gamma: 1,
        vibrance: 1
    },
    emphasis: {
        gamma: 1.2,
        vibrance: 1.3
    },
};
function styleIfNeed(foreground, background, styleSwatches) {
    if (foreground || background || styleSwatches) {
        return new style_1.Style(foreground, background, styleSwatches);
    }
    else {
        return undefined;
    }
}
function buildTheme(basePalette) {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r, _s, _t, _u, _v, _w, _x, _y, _z, _0, _1, _2, _3, _4, _5, _6, _7, _8, _9, _10, _11, _12, _13, _14, _15, _16, _17, _18, _19, _20, _21, _22, _23, _24, _25, _26, _27, _28, _29, _30, _31, _32, _33, _34, _35, _36, _37, _38, _39, _40, _41, _42, _43, _44, _45;
    const palette = {
        color: {
            remark: { r: 255, g: 128, b: 0 },
            value: { r: 208, g: 208, b: 208 },
            unvalue: { r: 180, g: 180, b: 120 },
            enumerable: {
                arm: rgbUtils_1.vibrance(rgbUtils_1.gammaCorrect(basePalette.enumerable.color, basePalette.dim.gamma), basePalette.dim.vibrance),
                iconForeground: basePalette.enumerable.counterColor,
                iconBackground: rgbUtils_1.vibrance(rgbUtils_1.gammaCorrect(basePalette.enumerable.color, basePalette.emphasis.gamma), basePalette.emphasis.vibrance),
                keyForeground: basePalette.enumerable.counterColor,
                keyBackground: basePalette.enumerable.color,
                keyDelimiter: basePalette.enumerable.color,
                info: rgbUtils_1.vibrance(rgbUtils_1.gammaCorrect(basePalette.enumerable.color, basePalette.dim.gamma), basePalette.dim.vibrance),
            },
            iterable: {
                arm: rgbUtils_1.vibrance(rgbUtils_1.gammaCorrect(basePalette.iterable.color, basePalette.dim.gamma), basePalette.dim.vibrance),
                iconForeground: basePalette.iterable.counterColor,
                iconBackground: rgbUtils_1.vibrance(rgbUtils_1.gammaCorrect(basePalette.iterable.color, basePalette.emphasis.gamma), basePalette.emphasis.vibrance),
                keyForeground: basePalette.iterable.counterColor,
                keyBackground: basePalette.iterable.color,
                keyDelimiter: basePalette.iterable.color,
                info: rgbUtils_1.vibrance(rgbUtils_1.gammaCorrect(basePalette.iterable.color, basePalette.dim.gamma), basePalette.dim.vibrance),
            },
            single: {
                arm: rgbUtils_1.vibrance(rgbUtils_1.gammaCorrect(basePalette.single.color, basePalette.dim.gamma), basePalette.dim.vibrance),
                iconForeground: basePalette.single.counterColor,
                iconBackground: rgbUtils_1.vibrance(rgbUtils_1.gammaCorrect(basePalette.single.color, basePalette.emphasis.gamma), basePalette.emphasis.vibrance),
                keyForeground: basePalette.single.counterColor,
                keyBackground: basePalette.single.color,
                keyDelimiter: basePalette.single.color,
                info: rgbUtils_1.vibrance(rgbUtils_1.gammaCorrect(basePalette.single.color, basePalette.dim.gamma), basePalette.dim.vibrance),
            },
            dead: {
                arm: rgbUtils_1.vibrance(rgbUtils_1.gammaCorrect(basePalette.dead.color, basePalette.dim.gamma), basePalette.dim.vibrance),
                iconForeground: basePalette.dead.counterColor,
                iconBackground: rgbUtils_1.vibrance(rgbUtils_1.gammaCorrect(basePalette.dead.color, basePalette.emphasis.gamma), basePalette.emphasis.vibrance),
                keyForeground: basePalette.dead.counterColor,
                keyBackground: basePalette.dead.color,
                keyDelimiter: basePalette.dead.color,
                info: rgbUtils_1.vibrance(rgbUtils_1.gammaCorrect(basePalette.dead.color, basePalette.dim.gamma), basePalette.dim.vibrance),
                elipsis: { r: 80, g: 80, b: 80 },
            },
        },
    };
    const theme = {
        global: {
            visibility: true,
            arm: {
                width: { preSpace: 1, arm: 3, postSpace: 0 },
            },
            icon: {
                prefix: {
                    line: ' ',
                },
                postfix: {
                    line: ' ',
                },
            },
            key: {
                prefix: {
                    line: ' ',
                },
                postfix: {
                    line: ' ',
                },
                postdelimiter: {
                    line: ': ',
                }
            },
            value: {
                style: styleIfNeed((_a = palette.color) === null || _a === void 0 ? void 0 : _a.value),
            },
            info: {
                predelimiter: {
                    line: ' · ',
                },
            },
            remark: {
                predelimiter: {
                    line: ' · ',
                },
                style: styleIfNeed((_b = palette.color) === null || _b === void 0 ? void 0 : _b.remark),
            },
        },
        broad: {
            enumerable: {
                arm: {
                    commonChars: ['├──', '│  ', '╰──',],
                    style: styleIfNeed((_d = (_c = palette.color) === null || _c === void 0 ? void 0 : _c.enumerable) === null || _d === void 0 ? void 0 : _d.arm),
                },
                info: {
                    style: styleIfNeed((_f = (_e = palette.color) === null || _e === void 0 ? void 0 : _e.enumerable) === null || _f === void 0 ? void 0 : _f.info, undefined, (_g = palette.styleSwatches) === null || _g === void 0 ? void 0 : _g.info),
                },
                icon: {
                    style: styleIfNeed((_j = (_h = palette.color) === null || _h === void 0 ? void 0 : _h.enumerable) === null || _j === void 0 ? void 0 : _j.iconForeground, (_l = (_k = palette.color) === null || _k === void 0 ? void 0 : _k.enumerable) === null || _l === void 0 ? void 0 : _l.iconBackground),
                },
                key: {
                    style: styleIfNeed((_o = (_m = palette.color) === null || _m === void 0 ? void 0 : _m.enumerable) === null || _o === void 0 ? void 0 : _o.keyForeground, (_q = (_p = palette.color) === null || _p === void 0 ? void 0 : _p.enumerable) === null || _q === void 0 ? void 0 : _q.keyBackground, (_r = palette.styleSwatches) === null || _r === void 0 ? void 0 : _r.key),
                    postdelimiter: {
                        style: styleIfNeed((_t = (_s = palette.color) === null || _s === void 0 ? void 0 : _s.enumerable) === null || _t === void 0 ? void 0 : _t.keyDelimiter),
                    }
                },
            },
            iterable: {
                arm: {
                    commonChars: ['├─━', '│  ', '└─━',],
                    style: styleIfNeed((_v = (_u = palette.color) === null || _u === void 0 ? void 0 : _u.iterable) === null || _v === void 0 ? void 0 : _v.arm),
                },
                info: {
                    style: styleIfNeed((_x = (_w = palette.color) === null || _w === void 0 ? void 0 : _w.iterable) === null || _x === void 0 ? void 0 : _x.info, undefined, (_y = palette.styleSwatches) === null || _y === void 0 ? void 0 : _y.info),
                },
                icon: {
                    style: styleIfNeed((_0 = (_z = palette.color) === null || _z === void 0 ? void 0 : _z.iterable) === null || _0 === void 0 ? void 0 : _0.iconForeground, (_2 = (_1 = palette.color) === null || _1 === void 0 ? void 0 : _1.iterable) === null || _2 === void 0 ? void 0 : _2.iconBackground),
                },
                key: {
                    style: styleIfNeed((_4 = (_3 = palette.color) === null || _3 === void 0 ? void 0 : _3.iterable) === null || _4 === void 0 ? void 0 : _4.keyForeground, (_6 = (_5 = palette.color) === null || _5 === void 0 ? void 0 : _5.iterable) === null || _6 === void 0 ? void 0 : _6.keyBackground, (_7 = palette.styleSwatches) === null || _7 === void 0 ? void 0 : _7.key),
                    postdelimiter: {
                        style: styleIfNeed((_9 = (_8 = palette.color) === null || _8 === void 0 ? void 0 : _8.iterable) === null || _9 === void 0 ? void 0 : _9.keyDelimiter),
                    }
                },
            },
            single: {
                arm: {
                    commonChars: ['├─╴', '│  ', '└─╴',],
                    style: styleIfNeed((_11 = (_10 = palette.color) === null || _10 === void 0 ? void 0 : _10.single) === null || _11 === void 0 ? void 0 : _11.arm),
                },
                info: {
                    style: styleIfNeed((_13 = (_12 = palette.color) === null || _12 === void 0 ? void 0 : _12.single) === null || _13 === void 0 ? void 0 : _13.info, undefined, (_14 = palette.styleSwatches) === null || _14 === void 0 ? void 0 : _14.info),
                },
                icon: {
                    style: styleIfNeed((_16 = (_15 = palette.color) === null || _15 === void 0 ? void 0 : _15.single) === null || _16 === void 0 ? void 0 : _16.iconForeground, (_18 = (_17 = palette.color) === null || _17 === void 0 ? void 0 : _17.single) === null || _18 === void 0 ? void 0 : _18.iconBackground),
                },
                key: {
                    style: styleIfNeed((_20 = (_19 = palette.color) === null || _19 === void 0 ? void 0 : _19.single) === null || _20 === void 0 ? void 0 : _20.keyForeground, (_22 = (_21 = palette.color) === null || _21 === void 0 ? void 0 : _21.single) === null || _22 === void 0 ? void 0 : _22.keyBackground, (_23 = palette.styleSwatches) === null || _23 === void 0 ? void 0 : _23.key),
                    postdelimiter: {
                        style: styleIfNeed((_25 = (_24 = palette.color) === null || _24 === void 0 ? void 0 : _24.single) === null || _25 === void 0 ? void 0 : _25.keyDelimiter),
                    }
                },
            },
            dead: {
                arm: {
                    commonChars: ['├─╴', '│  ', '╰─╴',],
                    style: styleIfNeed((_27 = (_26 = palette.color) === null || _26 === void 0 ? void 0 : _26.dead) === null || _27 === void 0 ? void 0 : _27.arm),
                },
                info: {
                    style: styleIfNeed((_29 = (_28 = palette.color) === null || _28 === void 0 ? void 0 : _28.dead) === null || _29 === void 0 ? void 0 : _29.info, undefined, (_30 = palette.styleSwatches) === null || _30 === void 0 ? void 0 : _30.info),
                },
                icon: {
                    style: styleIfNeed((_32 = (_31 = palette.color) === null || _31 === void 0 ? void 0 : _31.dead) === null || _32 === void 0 ? void 0 : _32.iconForeground, (_34 = (_33 = palette.color) === null || _33 === void 0 ? void 0 : _33.dead) === null || _34 === void 0 ? void 0 : _34.iconBackground),
                },
                key: {
                    style: styleIfNeed((_36 = (_35 = palette.color) === null || _35 === void 0 ? void 0 : _35.dead) === null || _36 === void 0 ? void 0 : _36.keyForeground, (_38 = (_37 = palette.color) === null || _37 === void 0 ? void 0 : _37.dead) === null || _38 === void 0 ? void 0 : _38.keyBackground, (_39 = palette.styleSwatches) === null || _39 === void 0 ? void 0 : _39.key),
                    postdelimiter: {
                        style: styleIfNeed((_41 = (_40 = palette.color) === null || _40 === void 0 ? void 0 : _40.dead) === null || _41 === void 0 ? void 0 : _41.keyDelimiter),
                    }
                },
            },
        },
        fine: {
            dead: {
                circularReference: {
                    icon: {
                        content: {
                            line: '∞',
                        }
                    }
                },
                elipsis: {
                    icon: {
                        prefix: {
                            line: '',
                        },
                        content: {
                            line: '...',
                        },
                        postfix: {
                            line: '',
                        },
                        style: styleIfNeed((_43 = (_42 = palette.color) === null || _42 === void 0 ? void 0 : _42.dead) === null || _43 === void 0 ? void 0 : _43.elipsis),
                    }
                },
            },
            single: {
                bigInt: {
                    icon: {
                        content: {
                            line: 'I',
                        }
                    }
                },
                boolean: {
                    icon: {
                        content: {
                            line: 'B',
                        }
                    }
                },
                booleanObject: {
                    icon: {
                        content: {
                            line: 'B',
                        }
                    }
                },
                date: {
                    icon: {
                        content: {
                            line: 'D',
                        }
                    }
                },
                function: {
                    icon: {
                        content: {
                            line: 'F',
                        }
                    }
                },
                null: {
                    icon: {
                        content: {
                            line: 'X',
                        }
                    },
                    value: {
                        style: styleIfNeed((_44 = palette.color) === null || _44 === void 0 ? void 0 : _44.unvalue)
                    }
                },
                number: {
                    icon: {
                        content: {
                            line: 'N',
                        }
                    }
                },
                numberObject: {
                    icon: {
                        content: {
                            line: 'N',
                        }
                    }
                },
                string: {
                    icon: {
                        content: {
                            line: 'T',
                        }
                    }
                },
                stringObject: {
                    icon: {
                        content: {
                            line: 'T',
                        }
                    }
                },
                symbol: {
                    icon: {
                        content: {
                            line: 'S',
                        }
                    }
                },
                undefined: {
                    icon: {
                        content: {
                            line: ' ',
                        }
                    },
                    value: {
                        style: styleIfNeed((_45 = palette.color) === null || _45 === void 0 ? void 0 : _45.unvalue)
                    }
                },
                unknown: {
                    icon: {
                        content: {
                            line: '?',
                        }
                    }
                },
                weakMap: {
                    icon: {
                        content: {
                            line: 'M',
                        }
                    }
                },
                weakSet: {
                    icon: {
                        content: {
                            line: 'S',
                        }
                    }
                },
            },
            enumerable: {
                map: {
                    icon: {
                        content: {
                            line: 'M',
                        }
                    }
                },
                object: {
                    icon: {
                        content: {
                            line: ' ',
                        },
                    }
                },
                error: {
                    icon: {
                        content: {
                            line: 'E',
                        }
                    }
                },
                unknown: {
                    icon: {
                        content: {
                            line: '?',
                        }
                    }
                },
            },
            iterable: {
                array: {
                    icon: {
                        content: {
                            line: ' ',
                        },
                    }
                },
                iterable: {
                    icon: {
                        content: {
                            line: 'I',
                        }
                    }
                },
                set: {
                    icon: {
                        content: {
                            line: 'S',
                        }
                    }
                },
                typedArray: {
                    icon: {
                        content: {
                            line: 'T',
                        }
                    }
                },
                unknown: {
                    icon: {
                        content: {
                            line: '?',
                        }
                    }
                },
            },
        },
    };
    return theme;
}
exports.plating = {
    standart: buildTheme(standartPalette),
    cute: buildTheme(cutePalette),
    alternative: buildTheme(alternativePalette),
};
//# sourceMappingURL=plating.js.map