"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const renderer_1 = require("../abstract/renderer");
const plain_1 = require("./plain");
const ansi_1 = require("./ansi");
const html_1 = require("./html");
class AutodetectRenderer extends renderer_1.Renderer {
    constructor() {
        super();
        this.features = this.detectFeatures();
        this.renderer = this.createRenderer();
        this.eol = this.renderer.eol;
    }
    detectFeatures() {
        // TODO: implement
        return {
            width: Infinity,
            env: 'unknown',
            allowStyles: false,
        };
    }
    createRenderer() {
        if (!this.features.allowStyles) {
            return new plain_1.PlainRenderer();
        }
        else {
            if (this.features.env === 'unknown') {
                return new plain_1.PlainRenderer();
            }
            else if (this.features.env === 'terminal') {
                return new ansi_1.AnsiRenderer();
            }
            else if (this.features.env === 'browser') {
                return new html_1.HtmlRenderer();
            }
            else {
                throw new Error('unsupported features env');
            }
        }
    }
    render(textContainer) {
        return this.renderer.render(textContainer);
    }
    renderFlat(flatTextContainer) {
        return this.renderer.renderFlat(flatTextContainer);
    }
    renderFlatFeedLines(flatFeedLines) {
        return this.renderer.renderFlatFeedLines(flatFeedLines);
    }
    renderFlatFeedLine(flatFeedLineContainer) {
        return this.renderer.renderFlatFeedLine(flatFeedLineContainer);
    }
    renderFlatLine(flatLineContainer) {
        return this.renderer.renderFlatLine(flatLineContainer);
    }
    renderLine(lineContainer) {
        return this.renderer.renderLine(lineContainer);
    }
}
exports.AutodetectRenderer = AutodetectRenderer;
//# sourceMappingURL=autodetect.js.map